"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class AutocompleteEntry {
    constructor() {
        this.output = "";
        this.label = "";
        this.documentation = "";
        this.isParameter = false;
        this.isSupeseding = false;
        this.isInternalVariable = false;
        this.isKeyword = false;
    }
    feedParameter(p) {
        this.isParameter = true;
        var parType = "* parameter";
        var values = [];
        if (p.type) {
            parType += "<" + p.type + ">";
        }
        if (p.validValues) {
            parType += "\n\tValid values:";
            p.validValues.forEach(function (vv) {
                if (typeof vv === "string" || typeof vv === "number") {
                    parType += "\n\t\t" + vv;
                    values.push(vv.toString());
                }
            });
        }
        if (p.validGroups) {
            parType += "\n\tValid groups:";
            p.validGroups.forEach(function (vg) {
                if (typeof vg === "string") {
                    parType += "\n\t\t" + vg;
                }
            });
        }
        if (p.valueObjects) {
            parType += "\n\tValue objects:";
            p.valueObjects.forEach(function (vo) {
                if (vo.value && typeof vo.value === "string") {
                    parType += "\n\t\t" + vo.value;
                    values.push(vo.value);
                    if (vo.labels && vo.labels.en) {
                        parType += " | " + vo.labels.en;
                    }
                }
            });
        }
        this.documentation = parType;
        this.label = p.key;
        this.output = p.key;
        return values;
    }
    feedSuperseding(subComponent, superseding) {
        if (superseding.key) {
            var supersedeDoc = "";
            if (this.isParameter || this.isSupeseding || this.isInternalVariable) {
                supersedeDoc = "\n";
            }
            else {
                this.label = superseding.key;
                this.output = superseding.key;
            }
            this.isSupeseding = true;
            if (subComponent.componentId && supersedeDoc.indexOf(subComponent.componentId) < 0) {
                supersedeDoc += "* superseded from " + subComponent.componentId;
            }
            this.documentation += supersedeDoc;
        }
    }
    feedInternalVariable(key) {
        var varDoc = "";
        if (!this.isInternalVariable) {
            if (this.isParameter || this.isSupeseding) {
                varDoc = "\n";
            }
            this.isInternalVariable = true;
            varDoc += "* internal variable";
        }
        this.label = key;
        this.output = key;
        this.documentation += varDoc;
    }
}
class Autocomplete {
    findAutocompleteEntries(obj, formattedComponent) {
        let identifiers = [
            Object.assign(new AutocompleteEntry(), { output: "userTriggeredChange", label: "parameter.onValueChange.userTriggeredChange", documenation: "Relevant in onValueChange script\n userTriggeredChange is true if onValueChange call cause was that user clicked a parameter.", isKeyword: true }),
            Object.assign(new AutocompleteEntry(), { output: "connection.isPreview", label: "connection.isPreview", documenation: "Is true if the docking point is in the preview phase. False if the connection is already docked.", isKeyword: true })
        ];
        if (obj.parameters) {
            obj.parameters.forEach(function (p) {
                if (p.key) {
                    var foundParameter = identifiers.find(el => el.output === p.key);
                    if (foundParameter === undefined) {
                        foundParameter = new AutocompleteEntry();
                        identifiers.push(foundParameter);
                    }
                    var parameterValues = foundParameter.feedParameter(p);
                    var shouldContainApostrophes = false;
                    if (p.type && "string" === typeof p.type) {
                        if (['string', 'material'].indexOf(p.type.toLowerCase()) > -1) {
                            shouldContainApostrophes = true;
                        }
                    }
                    parameterValues.forEach((value) => {
                        var outputValue = value;
                        if (shouldContainApostrophes) {
                            outputValue = "'" + outputValue + "'";
                        }
                        var valueEntry = Object.assign(new AutocompleteEntry(), {
                            output: outputValue,
                            label: p.key + "." + value
                        });
                        identifiers.push(valueEntry);
                    });
                }
            });
        }
        if (obj.subComponents) {
            obj.subComponents.forEach(function (subComponent) {
                if (subComponent.supersedings) {
                    subComponent.supersedings.forEach(function (superseding) {
                        var foundSuperseding = identifiers.find(el => el.output === superseding.key);
                        if (foundSuperseding === undefined) {
                            foundSuperseding = new AutocompleteEntry();
                            identifiers.push(foundSuperseding);
                        }
                        foundSuperseding.feedSuperseding(subComponent, superseding);
                    });
                }
            });
        }
        var matches = formattedComponent.match(/[a-zA-Z0-9_\.]*\s=\s/g);
        if (matches) {
            matches.forEach(function (s) {
                s = s.substring(0, s.length - 3);
                var foundVar = identifiers.find(el => el.output === s);
                if (foundVar === undefined) {
                    foundVar = new AutocompleteEntry();
                    identifiers.push(foundVar);
                }
                foundVar.feedInternalVariable(s);
            });
        }
        return identifiers;
    }
}
exports.Autocomplete = Autocomplete;
//# sourceMappingURL=autocomplete.js.map