"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const formatter = require("./formatter");
const autocomplete = require("./autocomplete");
const escaper = require("./escaper");
const massformatter = require("./massformatter");
let escButton;
let unescButton;
let formatAndFeedAutocompleteButton;
let formatButton;
let massFormatCancelButton;
let ac;
let f;
let esc;
let a;
let mf;
let formatterWorkingState = false;
function activate({ subscriptions }) {
    f = new formatter.Formatter();
    a = new autocomplete.Autocomplete();
    esc = new escaper.Escaper();
    mf = new massformatter.MassFormatter();
    console.log('RML Formatter 0.4.2 activating');
    escButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 200);
    escButton.command = 'extension.jsonMultilineEscape';
    escButton.text = '+EscML';
    massFormatCancelButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 200);
    massFormatCancelButton.command = 'extension.cancelFormatting';
    massFormatCancelButton.text = 'STOP MASS FORMATTING';
    unescButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 199);
    unescButton.command = 'extension.jsonMultilineUnEscape';
    unescButton.text = '-EscML';
    formatButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 198);
    formatButton.command = 'extension.formatRoomleComponent';
    formatButton.text = 'RML-Format+AutoComplete';
    formatAndFeedAutocompleteButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 198);
    formatAndFeedAutocompleteButton.command = 'extension.formatNoAutocomplete';
    formatAndFeedAutocompleteButton.text = 'RML-Format';
    setWorkingState(false);
    let autoCompleteProvider = vscode.languages.registerCompletionItemProvider('json', {
        provideCompletionItems(document, position, token, context) {
            return null;
        }
    });
    ac = autoCompleteProvider;
    subscriptions.push(autoCompleteProvider);
    subscriptions.push(escButton);
    subscriptions.push(unescButton);
    subscriptions.push(formatAndFeedAutocompleteButton);
    vscode.window.onDidChangeActiveTextEditor((textEditor) => {
        if (!textEditor) {
            return;
        }
        if (textEditor && textEditor.document.languageId === 'json') {
            runFormatterAndAutocomplete(true, false);
        }
    });
    vscode.workspace.onDidOpenTextDocument((textDocument) => {
        if (!formatterWorkingState && textDocument && textDocument.languageId === 'json') {
            runFormatterAndAutocomplete(true, false);
        }
    });
    let disposableFormatAndFeedAutocomplete = vscode.commands.registerCommand('extension.formatRoomleComponent', () => {
        if (!formatterWorkingState) {
            runFormatterAndAutocomplete(true, true);
        }
    });
    let disposableFormat = vscode.commands.registerCommand('extension.formatNoAutocomplete', () => {
        if (!formatterWorkingState) {
            runFormatterAndAutocomplete(false, true);
        }
    });
    let disposableFormatAll = vscode.commands.registerCommand('extension.formatAllOpenRoomleJsons', () => {
        if (!formatterWorkingState) {
            formatAllTabs();
        }
    });
    let disposableFormatOverride = vscode.commands.registerCommand('extension.overrideFormatWhileAllTabs', () => {
        runFormatterAndAutocomplete(false, true);
    });
    let disposableCancelFormatting = vscode.commands.registerCommand('extension.cancelFormatting', () => {
        f.shouldCancel = true;
        mf.shouldCancel = true;
    });
    let disposableEscape = vscode.commands.registerCommand('extension.jsonMultilineEscape', () => {
        if (vscode.window.activeTextEditor && !formatterWorkingState) {
            var escaped = esc.getEscaped(getDocument());
            apply(escaped);
            vscode.window.showInformationMessage("Escaped");
        }
    });
    let disposableUnEscape = vscode.commands.registerCommand('extension.jsonMultilineUnEscape', () => {
        if (vscode.window.activeTextEditor && !formatterWorkingState) {
            var unescaped = esc.getUnescaped(getDocument());
            apply(unescaped);
            vscode.window.showInformationMessage("Unescaped");
        }
    });
    function runFormatterAndAutocomplete(feedAutocomplete, applyFormatting) {
        if (!feedAutocomplete && !applyFormatting) {
            vscode.window.showWarningMessage("Request not to format neither feed autocomplete. This should not happen");
        }
        if (vscode.window.activeTextEditor) {
            var escaped = esc.getEscaped(getDocument());
            var clean = escaped.replace(/\r?\n|\r/g, '');
            var obj = JSON.parse(clean);
            obj = f.formatRoomleJson(obj);
            if (f.shouldCancel) {
                f.shouldCancel = false;
                vscode.window.showWarningMessage("Job's cancelled");
            }
            else {
                var formated = JSON.stringify(obj, null, 4);
                var unescaped = esc.getUnescaped(formated);
                if (feedAutocomplete) {
                    let identifiers = a.findAutocompleteEntries(obj, formated);
                    let autoCompleteProvider = vscode.languages.registerCompletionItemProvider('json', {
                        provideCompletionItems(document, position, token, context) {
                            let simpleCompletions = [];
                            identifiers.forEach(id => {
                                const simpleCompletion = new vscode.CompletionItem(id.label);
                                simpleCompletion.insertText = id.output;
                                simpleCompletion.documentation = id.documentation;
                                simpleCompletions.push(simpleCompletion);
                            });
                            return simpleCompletions;
                        }
                    });
                    ac.dispose();
                    ac = autoCompleteProvider;
                    subscriptions.push(autoCompleteProvider);
                }
                if (applyFormatting) {
                    apply(unescaped);
                }
                vscode.window.showInformationMessage("Job's done");
            }
        }
    }
    function setWorkingState(working) {
        if (working) {
            massFormatCancelButton.show();
            escButton.hide();
            unescButton.hide();
            formatAndFeedAutocompleteButton.hide();
            formatButton.hide();
        }
        else {
            massFormatCancelButton.hide();
            escButton.show();
            unescButton.show();
            formatAndFeedAutocompleteButton.show();
            formatButton.show();
            f.shouldCancel = true;
            mf.shouldCancel = true;
        }
        formatterWorkingState = working;
    }
    function getDocument() {
        if (vscode.window.activeTextEditor) {
            var json = vscode.window.activeTextEditor.document.getText();
            return json;
        }
        else {
            return "";
        }
    }
    function apply(data) {
        if (vscode.window.activeTextEditor) {
            vscode.window.activeTextEditor.edit(editBuilder => {
                if (!vscode.window.activeTextEditor) {
                    return;
                }
                editBuilder.replace(new vscode.Range(0, 0, vscode.window.activeTextEditor.document.lineCount, 0), data);
            });
        }
    }
    function formatAllTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            setWorkingState(true);
            yield mf.formatAllFiles();
            setWorkingState(false);
        });
    }
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map