"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
class Formatter {
    constructor() {
        this.shouldCancel = false;
    }
    implementTable(currentObj, tableTriggerLine, split, index, indentLevel = 2) {
        var tableSplit = tableTriggerLine.substring(6).trim().replace(/\s/g, "").split(/[;,]/g);
        var params = [];
        if (tableSplit.length < 1) {
            vscode.window.showErrorMessage("TABLE was not implemented. No arguments");
            return split;
        }
        if (!currentObj.parameters) {
            vscode.window.showErrorMessage("TABLE was not implemented. Did not find parameters");
            return split;
        }
        /* Get list of parameters */
        for (var i = 0; i < tableSplit.length; i++) {
            var found = currentObj.parameters.find((p) => {
                if (p.key) {
                    return p.key === tableSplit[i];
                }
                else {
                    return false;
                }
            });
            if (found !== null) {
                params.push(found);
            }
            else {
                vscode.window.showErrorMessage("TABLE was not implemented. Did not find parameter with key " + tableSplit[i]);
                if (currentObj.id) {
                    vscode.window.showInformationMessage(currentObj.id);
                }
                return split;
            }
        }
        var table = this.writeTableParameter(params);
        vscode.window.showInformationMessage(index + "   " + table);
        var injection = [tableTriggerLine.replace("#", "/*") + "*/"];
        injection.push(table);
        injection.push(tableTriggerLine.replace("#", "/* END OF ") + "*/");
        var result = split.slice(0, index);
        result = result.concat(injection);
        result = result.concat(split.slice(index + 1));
        return result;
    }
    writeTableParameter(params) {
        var param = params[0];
        if (!param.key) {
            return "/* null */";
        }
        vscode.window.showInformationMessage(param.key);
        var quotes = "'";
        if (param.type) {
            if (["Decimal", "Integer"].indexOf(param.type) > -1) {
                quotes = "";
            }
        }
        var values = [];
        var result = "";
        if (param.type === "Boolean") {
            result = "if (" + param.key + ") {";
            if (params.length > 1) {
                result += "\n" + this.writeTableParameter(params.slice(1)) + "} else {";
            }
            else {
                result += "}\n else {";
            }
            if (params.length > 1) {
                result += "\n" + this.writeTableParameter(params.slice(1));
            }
            result += "}\n";
        }
        else {
            if (param.validValues) {
                values = param.validValues;
            }
            else if (param.valueObjects) {
                for (var j = 0; j < param.valueObjects.length; j++) {
                    if (param.valueObjects[j].value) {
                        values.push(param.valueObjects[j].value);
                    }
                }
            }
            else {
                vscode.window.showErrorMessage("TABLE was not implemented. Unless boolean, this requires parameters with validValues or valueObjects: " + param.key);
                return "/* null */";
            }
            var prefix = "if (" + param.key + " == " + quotes;
            var suffix = quotes + ") {";
            var closing = "}\n";
            values.forEach((value) => {
                result += prefix + value + suffix;
                if (params.length > 1) {
                    result += "\n" + this.writeTableParameter(params.slice(1));
                }
                result += closing;
            });
        }
        return result;
    }
    formatScript(currentObj, data, indentLevel = 2, path = "script") {
        if (this.shouldCancel) {
            return data;
        }
        var split = data.split("\n");
        var splitLength = split.length;
        var startIndentLevel = indentLevel;
        var firstEmptyLines = true;
        var numOfEmptyLines = 0;
        var indentersCounts = {};
        indentersCounts['{'] = 0;
        indentersCounts['['] = 0;
        indentersCounts['('] = 0;
        indentersCounts['ObjGroup'] = 0;
        var shownIndentersCountsError = {};
        shownIndentersCountsError['{'] = false;
        shownIndentersCountsError['['] = false;
        shownIndentersCountsError['('] = false;
        var thisLineIndent = indentLevel;
        var nextLineIndent = indentLevel;
        var insideComment = 0;
        for (var i = 0; i < splitLength && (!this.shouldCancel); i++) {
            var line = split[i].trim();
            if (line.startsWith("#TABLE")) {
                split = this.implementTable(currentObj, line, split, i, indentLevel);
                i += split.length - splitLength;
                splitLength = split.length;
                continue;
            }
            var moveUp = false;
            /* remove starting emptylines or wherever there are more than 2 */
            if (line === "") {
                if ((numOfEmptyLines >= 2 || firstEmptyLines)) {
                    split.splice(i, 1);
                    splitLength--;
                    i--;
                }
                numOfEmptyLines++;
            }
            else {
                firstEmptyLines = false;
                numOfEmptyLines = 0;
                var s = this.clearSpaces(line, insideComment);
                var sL = s.length;
                var atBeginning = true;
                thisLineIndent = nextLineIndent;
                var insideString = false;
                var r = "";
                var spaces = 0;
                var minusNotFollowedBySpace = false;
                var previousIsParenthesis = false;
                for (var j = 0; j < sL && (!this.shouldCancel); j++) {
                    var c = s.substr(j, 1);
                    var c1 = " ";
                    if (j < sL - 1) {
                        c1 = s.substr(j + 1, 1);
                    }
                    var cn = c + c1;
                    /* mark correct comments */
                    if (cn === "/*") {
                        r += "/*";
                        j++;
                        insideComment++;
                    }
                    else if (cn === "*/") {
                        r += "*/";
                        j++;
                        insideComment--;
                    }
                    else if ("'" === c) {
                        r += c;
                        if (insideComment < 1) {
                            insideString = !insideString;
                        }
                        spaces = 0;
                    }
                    else {
                        /* make everything between comments ignored */
                        if (insideComment < 1 && !insideString) {
                            if (["(", "{", "["].indexOf(c) > -1) {
                                nextLineIndent++;
                                if (previousIsParenthesis) {
                                    r = r.trimRight();
                                }
                                r += c;
                                /*r += c + " ";
                                r = r.trimRight()  + c;*/
                                spaces = 1;
                                indentersCounts[c]++;
                                minusNotFollowedBySpace = true;
                                previousIsParenthesis = true;
                            }
                            else if ([")", "}", "]"].indexOf(c) > -1) {
                                nextLineIndent--;
                                if (atBeginning) {
                                    thisLineIndent--;
                                }
                                r = r.trimRight() + c;
                                spaces = 0;
                                if (c === "}") {
                                    indentersCounts['{']--;
                                }
                                else if (c === ")") {
                                    indentersCounts['(']--;
                                }
                                else if (c === "]") {
                                    indentersCounts['[']--;
                                }
                                previousIsParenthesis = true;
                            }
                            else if (c === " ") {
                                if (spaces === 0) {
                                    r += " ";
                                    spaces++;
                                }
                                else {
                                    spaces++;
                                }
                            }
                            else if (["++", "--"].indexOf(cn) > -1) {
                                r = r.trimRight() + cn;
                                minusNotFollowedBySpace = false;
                                spaces = 0;
                                j++;
                                previousIsParenthesis = false;
                            }
                            else if (["==", ">=", "<=", "||", "!=", "&&"].indexOf(cn) > -1) {
                                r = r.trimRight() + " " + cn + " ";
                                spaces = 1;
                                j++;
                                previousIsParenthesis = false;
                                minusNotFollowedBySpace = false;
                                if (atBeginning) {
                                    moveUp = true;
                                }
                            }
                            else if ("-" === c) {
                                r = r.trimRight() + " -";
                                if (minusNotFollowedBySpace) {
                                    spaces = 0;
                                }
                                else {
                                    r += " ";
                                    spaces = 1;
                                }
                                if (atBeginning) {
                                    moveUp = true;
                                }
                                previousIsParenthesis = false;
                            }
                            else if (["+", "*", "/", "=", "|", ">", "<"].indexOf(c) > -1) {
                                r = r.trimRight() + " " + c + " ";
                                spaces = 1;
                                minusNotFollowedBySpace = c === "=";
                                if (atBeginning) {
                                    moveUp = true;
                                }
                                previousIsParenthesis = false;
                            }
                            else if ([","].indexOf(c) > -1) {
                                r = r.trimRight() + c + " ";
                                spaces = 1;
                                minusNotFollowedBySpace = true;
                                previousIsParenthesis = false;
                            }
                            else if ([";"].indexOf(c) > -1 && j < sL - 1) {
                                r = r.trimRight() + c + " ";
                                if (atBeginning) {
                                    moveUp = true;
                                }
                                spaces = 1;
                                minusNotFollowedBySpace = false;
                                previousIsParenthesis = false;
                            }
                            else {
                                r += c;
                                spaces = 0;
                                minusNotFollowedBySpace = false;
                                previousIsParenthesis = false;
                            }
                        }
                        else {
                            r += c;
                            spaces = 0;
                        }
                    }
                    atBeginning = false;
                }
                line = r
                    .replace(/if\(/g, "if (")
                    .replace(/for\(/g, "for (")
                    .replace(/\}else/g, "} else")
                    .replace(/else\{/g, "else {")
                    .replace(/else\s\{/g, "else { ")
                    .replace(/\)\{/g, ") {")
                    .replace(/\)\s\{/g, ") { ")
                    .replace(/e - 00/g, "e-00")
                    .replace(/;\}/g, "; }");
                line = line.trim();
                if (line.startsWith("MoveMatrixBy") || line.startsWith("RotateMatrixBy") || line.startsWith("SetObjSurface")) {
                    line = " " + line;
                }
                var beginObjGroupMatches = (line.match(/BeginObjGroup/g) || []).length;
                nextLineIndent += beginObjGroupMatches;
                indentersCounts['ObjGroup'] += beginObjGroupMatches;
                var endObjGroupMatches = (line.match(/EndObjGroup/g) || []).length;
                thisLineIndent -= endObjGroupMatches;
                nextLineIndent -= endObjGroupMatches;
                indentersCounts['ObjGroup'] -= endObjGroupMatches;
                if (line === "") {
                    split[i] = "";
                }
                else {
                    split[i] = " ".repeat(thisLineIndent * 4) + line.trimRight();
                }
                if (insideString) {
                    vscode.window.showErrorMessage(path + " line " + (i + 1) + ": odd number of apostrophes, check unclosed strings");
                    return data;
                }
                if (insideComment < 1 && indentersCounts['['] < 0 && !shownIndentersCountsError['[']) {
                    vscode.window.showErrorMessage(path + " line " + (i + 1) + ": there is a closing bracket, but it has nothing to close - []");
                    shownIndentersCountsError['['] = true;
                }
                if (insideComment < 1 && indentersCounts['('] < 0 && !shownIndentersCountsError['(']) {
                    vscode.window.showErrorMessage(path + " line " + (i + 1) + ": there is a closing parenthesis, but it has nothing to close - ()");
                    shownIndentersCountsError['('] = true;
                }
                if (insideComment < 1 && indentersCounts['{'] < 0 && !shownIndentersCountsError['{']) {
                    vscode.window.showErrorMessage(path + " line " + (i + 1) + ": there is a closing brace, but it has nothing to close - {}");
                    shownIndentersCountsError['{'] = true;
                }
            }
        }
        /* remove trailing emptylines */
        if (!firstEmptyLines) {
            firstEmptyLines = true;
            for (i = splitLength - 1; i > 0 && firstEmptyLines && (!this.shouldCancel); i--) {
                line = split[i].trim();
                if (line === "") {
                    split.splice(i, 1);
                    splitLength--;
                }
                else {
                    firstEmptyLines = false;
                }
            }
        }
        /* above code lets lines containing only whitespaces through, evne it should not - unfortunately inducing following hotfix */
        for (i = 0; i < splitLength && (!this.shouldCancel); i++) {
            line = split[i].trim();
            if (line === "") {
                split[i] = "";
            }
        }
        if (indentersCounts['['] !== 0) {
            vscode.window.showErrorMessage(path + ": There are " + indentersCounts['['] + " [ more than ] ");
        }
        if (indentersCounts['('] !== 0) {
            vscode.window.showErrorMessage(path + ": There are " + indentersCounts['('] + " ( more than ) ");
        }
        if (indentersCounts['{'] !== 0) {
            vscode.window.showErrorMessage(path + ": There are " + indentersCounts['{'] + " { more than } ");
        }
        if (indentersCounts['ObjGroup'] !== 0) {
            vscode.window.showErrorMessage(path + ": There are " + indentersCounts['ObjGroup'] + " BeginObjGroup more than EndObjGroup");
        }
        if (insideComment !== 0) {
            vscode.window.showInformationMessage(path + ": There are " + insideComment + " /* more than */ ");
        }
        if (this.shouldCancel) {
            return data;
        }
        else {
            return split.join("\n");
        }
    }
    clearSpaces(line, insideComment) {
        var sL = line.length;
        var insideString = false;
        var r = "";
        for (var j = 0; j < sL && (!this.shouldCancel); j++) {
            var c = line.substr(j, 1);
            var cn = "";
            if (j < sL - 1) {
                cn = c + line.substr(j + 1, 1);
            }
            if (cn === "//") {
                r += "/*";
                j++;
                line += "*/";
                sL += 2;
                insideComment++;
            }
            else if (cn === "/*") {
                r += "/*";
                j++;
                insideComment++;
            }
            else if (cn === "*/") {
                r += "*/";
                j++;
                insideComment--;
            }
            else if ("'" === c) {
                r += c;
                insideString = !insideString;
            }
            else {
                if (insideComment < 1 && !insideString) {
                    if (c !== " ") {
                        r += c;
                    }
                }
                else {
                    r += c;
                }
            }
        }
        return r;
    }
    formatRoomleJson(comp) {
        this.shouldCancel = false;
        if (comp.onUpdate) {
            comp.onUpdate = this.format(comp, comp.onUpdate, 2, "onUpdate");
        }
        if (comp.parameterGroups) {
            comp.parameterGroups.forEach((pg) => {
                var pgKey = "";
                if (pg.key) {
                    pgKey = ":" + pg.key;
                }
                if (pg.collapsed && "string" === typeof pg.collapsed) {
                    pg.collapsed = this.format(comp, pg.collapsed, 4, "groups[" + comp.parameterGroups.indexOf(pg) + pgKey + "]/");
                }
            });
        }
        if (comp.parameters) {
            comp.parameters.forEach((p) => {
                var pIndex = comp.parameters.indexOf(p);
                var pKey = "";
                if (p.key) {
                    pKey = ":" + p.key;
                }
                if (p.onValueChange && typeof p.onValueChange === "string") {
                    p.onValueChange = this.format(comp, p.onValueChange, 4, "parameters[" + pIndex + pKey + "]/onValueChange");
                }
                if (p.enabled && typeof p.enabled === "string") {
                    p.enabled = this.format(comp, p.enabled, 4, "parameters[" + pIndex + pKey + "]/enabled");
                }
                if (p.visible && typeof p.visible === "string") {
                    p.visible = this.format(comp, p.visible, 4, "parameters[" + pIndex + pKey + "]/visible");
                }
                if (p.enabled && typeof p.enabled === "string") {
                    p.enabled = this.format(comp, p.enabled, 4, "parameters[" + pIndex + pKey + "]/enabled");
                }
                if (p.visibleAsGlobal && typeof p.visibleAsGlobal === "string") {
                    p.visibleAsGlobal = this.format(comp, p.visibleAsGlobal, 4, "parameters[" + pIndex + pKey + "]/visibleAsGlobal");
                }
                if (p.visibleInPartList && typeof p.visibleInPartList === "string") {
                    p.visibleInPartList = this.format(comp, p.visibleInPartList, 4, "parameters[" + pIndex + pKey + "]/visibleInPartList");
                }
                if (p.highlighted && typeof p.highlighted === "string") {
                    p.highlighted = this.format(comp, p.highlighted, 4, "parameters[" + pIndex + pKey + "]/highlighted");
                }
                if (p.valueObjects) {
                    p.valueObjects.forEach((vo) => {
                        if (vo.condition && typeof vo.condition === "string") {
                            vo.condition = this.format(comp, vo.condition, 6, "parameters[" + pIndex + pKey + "]/valueObjects[" + p.valueObjects.indexOf(vo) + "]/condition");
                        }
                    });
                }
                if (p.validRange) {
                    if (p.validRange.step && typeof p.validRange.step === "string") {
                        p.validRange.step = this.format(comp, p.validRange.step, 4, "parameters[" + pIndex + pKey + "]/validRange/step");
                    }
                    if (p.validRange.valueFrom && typeof p.validRange.valueFrom === "string") {
                        p.validRange.valueFrom = this.format(comp, p.validRange.valueFrom, 4, "parameters[" + pIndex + pKey + "]/validRange/valueFrom");
                    }
                    if (p.validRange.valueTo && typeof p.validRange.valueTo === "string") {
                        p.validRange.valueTo = this.format(comp, p.validRange.valueTo, 4, "parameters[" + pIndex + pKey + "]/validRange/valueTo");
                    }
                }
            });
        }
        if (comp.subComponents) {
            comp.subComponents.forEach((sc) => {
                var scId = "";
                if (sc.internalId) {
                    scId = ":" + sc.internalId;
                }
                if (sc.active && "string" === typeof sc.active) {
                    sc.active = this.format(comp, sc.active, 4, "subComponents[" + comp.subComponents.indexOf(sc) + scId + "]/active");
                }
                if (sc.numberInPartList && "string" === typeof sc.numberInPartList) {
                    sc.numberInPartList = this.format(comp, sc.numberInPartList, 4, "subComponents[" + comp.subComponents.indexOf(sc) + scId + "]/numberInPartList");
                }
                if (sc.assignments) {
                    /*var keys = Object.keys(sc.assignments);
                    keys.forEach(function (key: any) {
                        if (sc.assignments[key]) {
                            sc.assignments[key] = this.format(comp, sc.assignments[key], 5, "subComponents[" + c.subComponents.indexOf(sc) + "]/assignments/" + key);
                        }
                    });*/
                    sc.assignments = this.formatAssignments(comp, sc.assignments, 5, "subComponents[" + comp.subComponents.indexOf(sc) + scId + "]/assignments/");
                }
            });
        }
        if (comp.addOnSpots) {
            comp.addOnSpots.forEach((aos) => {
                if (aos.position && "string" === typeof aos.position) {
                    this.format(comp, aos.position, 4, "addOnSpots[" + comp.subComponents.indexOf(aos) + "]/position");
                }
                if (aos.condition && "string" === typeof aos.condition) {
                    this.format(comp, aos.condition, 4, "addOnSpots[" + comp.subComponents.indexOf(aos) + "]/position");
                }
            });
        }
        if (comp.geometry && typeof comp.geometry === "string") {
            comp.geometry = this.format(comp, comp.geometry, 2, "geometry");
        }
        if (comp.articleNr && typeof comp.articleNr === "string") {
            comp.articleNr = this.format(comp, comp.articleNr, 2, "articleNr");
        }
        if (comp.label && typeof comp.label === "string") {
            comp.label = this.format(comp, comp.label, 2, "label");
        }
        if (comp.previewGeometry && typeof comp.previewGeometry === "string") {
            comp.previewGeometry = this.format(comp, comp.previewGeometry, 2, "previewGeometry");
        }
        if (comp.geometryHD && typeof comp.geometryHD === "string") {
            comp.geometryHD = this.format(comp, comp.geometryHD, 2, "geometryHD");
        }
        if (comp.valid && typeof comp.valid === "string") {
            comp.valid = this.format(comp, comp.valid, 2, "valid");
        }
        if (comp.siblings) {
            comp.siblings.forEach((s) => { this.formatConnection(comp, s, 4, "siblings[" + comp.siblings.indexOf(s) + "]/"); });
        }
        if (comp.parentDockings) {
            if (comp.parentDockings.points) {
                comp.parentDockings.points.forEach((conn) => { this.formatConnection(comp, conn, 5, "parentDockings/points[" + comp.parentDockings.points.indexOf(conn) + "]/"); });
            }
            if (comp.parentDockings.lines) {
                comp.parentDockings.lines.forEach((conn) => { this.formatConnection(comp, conn, 5, "parentDockings/lines[" + comp.parentDockings.lines.indexOf(conn) + "]/"); });
            }
            if (comp.parentDockings.ranges) {
                comp.parentDockings.ranges.forEach((conn) => { this.formatConnection(comp, conn, 5, "parentDockings/ranges[" + comp.parentDockings.ranges.indexOf(conn) + "]/"); });
            }
        }
        if (comp.pricing) {
            comp.pricing.forEach((pc) => {
                if (pc.retailPriceDependsOnCustomerPrice && "string" === typeof pc.retailPriceDependsOnCustomerPrice) {
                    pc.retailPriceDependsOnCustomerPrice = this.format(comp, pc.retailPriceDependsOnCustomerPrice, 4, "pricing[" + comp.pricing.indexOf(pc) + "]/retailPriceDependsOnCustomerPrice");
                }
                if (pc.price && "string" === typeof pc.price) {
                    pc.price = this.format(comp, pc.price, 4, "pricing[" + comp.pricing.indexOf(pc) + "]/price");
                }
                if (pc.retailPrice && "string" === typeof pc.retailPrice) {
                    pc.retailPrice = this.format(comp, pc.retailPrice, 4, "pricing[" + comp.pricing.indexOf(pc) + "]/retailPrice");
                }
            });
        }
        if (comp.packaging) {
            comp.packaging.forEach((p) => {
                if (p.condition && typeof p.condition === "string") {
                    p.condition = this.format(comp, p.condition, 4, "packaging[" + comp.packaging.indexOf(p) + "]/condition");
                }
            });
        }
        if (comp.dimensioning) {
            comp.dimensioning.forEach((d) => {
                if (d.from && typeof d.from === "string") {
                    d.from = this.format(comp, d.from, 4, "dimensioning[" + comp.pricing.indexOf(d) + "]/from");
                }
                if (d.to && typeof d.to === "string") {
                    d.to = this.format(comp, d.to, 4, "dimensioning[" + comp.pricing.indexOf(d) + "]/to");
                }
                if (d.visible && typeof d.visible === "string") {
                    d.visible = this.format(comp, d.visible, 4, "dimensioning[" + comp.pricing.indexOf(d) + "]/visible");
                }
            });
        }
        if (comp.childDockings) {
            if (comp.childDockings.points) {
                comp.childDockings.points.forEach((conn) => { this.formatConnection(comp, conn, 5, "childDockings/points[" + comp.childDockings.points.indexOf(conn) + "]/"); });
            }
        }
        if (comp.possibleChildren) {
            comp.possibleChildren.forEach((pc) => {
                if (pc.condition && typeof pc.condition === "string") {
                    pc.condition = this.format(comp, pc.condition, 4, "possibleChildren[" + comp.possibleChildren.indexOf(pc) + "]/condition");
                }
                if (pc.default && typeof pc.default === "string") {
                    pc.default = this.format(comp, pc.default, 4, "possibleChildren[" + comp.possibleChildren.indexOf(pc) + "]/default");
                }
            });
        }
        return comp;
    }
    formatConnection(currentObj, c, indent = 0, path = "") {
        if (c.mask) {
            path = path.substr(0, path.length - 2) + ":" + c.mask + "]/";
        }
        if (c.position && typeof c.position === "string") {
            c.position = this.format(currentObj, c.position, indent, path + "position");
        }
        if (c.rotation && typeof c.rotation === "string") {
            c.rotation = this.format(currentObj, c.rotation, indent, path + "rotation");
        }
        if (c.positionTo && typeof c.positionTo === "string") {
            c.positionTo = this.format(currentObj, c.positionTo, indent, path + "positionTo");
        }
        if (c.condition && typeof c.condition === "string") {
            c.condition = this.format(currentObj, c.condition, indent, path + "condition");
        }
        if (c.stepX && typeof c.stepX === "string") {
            c.stepX = this.format(currentObj, c.stepX, indent, path + "stepX");
        }
        if (c.stepY && typeof c.stepY === "string") {
            c.stepY = this.format(currentObj, c.stepY, indent, path + "stepY");
        }
        if (c.stepZ && typeof c.stepZ === "string") {
            c.stepZ = this.format(currentObj, c.stepZ, indent, path + "stepZ");
        }
        if (c.stepEnd && typeof c.stepEnd === "string") {
            c.stepEnd = this.format(currentObj, c.stepEnd, indent, path + "stepEnd");
        }
        if (c.maxConnections && typeof c.maxConnections === "string") {
            c.maxConnections = this.format(currentObj, c.maxConnections, indent, path + "maxConnections");
        }
        if (c.assignmentScripts) {
            if (c.assignmentScripts.onDock && typeof c.assignmentScripts.onDock === "string") {
                c.assignmentScripts.onDock = this.format(currentObj, c.assignmentScripts.onDock, indent + 1, path + "assignmentScripts/onDock");
            }
            if (c.assignmentScripts.onUnDock && typeof c.assignmentScripts.onUnDock === "string") {
                c.assignmentScripts.onUnDock = this.format(currentObj, c.assignmentScripts.onUnDock, indent + 1, path + "assignmentScripts/onUnDock");
            }
            if (c.assignmentScripts.onUpdate && typeof c.assignmentScripts.onUpdate === "string") {
                c.assignmentScripts.onUpdate = this.format(currentObj, c.assignmentScripts.onUpdate, indent + 1, path + "assignmentScripts/onUpdate");
            }
        }
        if (c.selfAssignments) {
            if (c.selfAssignments.onDock) {
                c.selfAssignments.onDock = this.formatAssignments(currentObj, c.selfAssignments.onDock, indent + 2, path + "selfAssignments/onDock/");
            }
            if (c.selfAssignments.onUnDock) {
                c.selfAssignments.onUnDock = this.formatAssignments(currentObj, c.selfAssignments.onUnDock, indent + 2, path + "selfAssignments/onUnDock/");
            }
            if (c.selfAssignments.onUpdate) {
                c.selfAssignments.onUpdate = this.formatAssignments(currentObj, c.selfAssignments.onUpdate, indent + 2, path + "selfAssignments/onUpdate/");
            }
        }
        if (c.assignmentsOnDock) {
            c.assignmentsOnDock = this.formatAssignments(currentObj, c.assignmentsOnDock, indent + 1, path + "assignmentsOnDock/");
        }
        if (c.assignmentsOnUpdate) {
            c.assignmentsOnUpdate = this.formatAssignments(currentObj, c.assignmentsOnUpdate, indent + 1, path + "assignmentsOnUpdate/");
        }
        if (c.assignmentsOnUpdateSilent) {
            c.assignmentsOnUpdateSilent = this.formatAssignments(currentObj, c.assignmentsOnUpdateSilent, indent + 1, path + "assignmentsOnUpdateSilent/");
        }
        if (c.assignmentsOnUnDock) {
            c.assignmentsOnUnDock = this.formatAssignments(currentObj, c.assignmentsOnUnDock, indent + 1, path + "assignmentsOnUnDock/");
        }
        return c;
    }
    formatAssignments(currentObj, data, indentation = 2, path) {
        if (data) {
            var keys = Object.keys(data);
            keys.forEach((key) => {
                if (data[key] && typeof data[key] === "string") {
                    data[key] = this.format(currentObj, data[key], indentation, path + "/" + key);
                }
            });
        }
        return data;
    }
    format(currentObj, data, indentLevel = 2, path = "script") {
        if (data.indexOf("\n") > -1) {
            data = this.formatScript(currentObj, data, indentLevel, path);
            if (data.length > 0) {
                if (!data.startsWith("\n")) {
                    data = "\n" + data;
                }
                if (!data.endsWith("\n" + " ".repeat((indentLevel - 1) * 4))) {
                    data = data + "\n" + " ".repeat((indentLevel - 1) * 4);
                }
            }
        }
        else {
            data = this.formatScript(currentObj, data, indentLevel, path).trim();
        }
        return data;
    }
}
exports.Formatter = Formatter;
//# sourceMappingURL=formatter.js.map